@@ -1,8 +1,12 @@
 from typing import Dict
+
 import platform
 import psutil
 
 
+IS_WINDOWS = platform.system() == "Windows"
+
+
 def get_process_list() -> Dict:
     """Get process list.
 
@@ -23,10 +27,6 @@ def is_numeric(value):
         return float(value).is_integer()
 
 
-def is_windows():
-    return platform.system() == "Windows"
-
-
 def call_attribute_if_available(object_name, attribute_name):
     if hasattr(object_name, attribute_name):
         return getattr(object_name, attribute_name)()
